/* 
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  


   ### File Specific:
   main-programmer: Benjamin Grauer

*/

#include "orxonox_gui_exec.h"
#include <iostream>
#include <string>

/** 
    \brief Creates the Exec-Frame
    \param orxonoxGUI ExecFrame needs to know where to get the Options from
*/
OrxonoxGuiExec::OrxonoxGuiExec (Window* orxonoxGUI)
{
  configFile = (char*)malloc (512*sizeof (char));

  execFrame = new Frame ("Execute-Tags:");
  execBox = new Box ('v');
  execFrame->setGroupName ("misc");
  
  start = new Button ("Start");
  start->connectSignal ("clicked", startOrxonox);
  execBox->fill (start);
  saveSettings = new CheckButton ("Save Settings");
  saveSettings->value = 1;
  execBox->fill (saveSettings);
  verboseMode = new Menu ("verbose mode", "no output", "verbose", "debug", "lastItem");
  verboseMode->setFlagName ("verbose", "v", 0);
  execBox->fill (verboseMode);
  alwaysShow = new CheckButton ("Always Show this Menu");
  alwaysShow->setFlagName ("gui", "g", 0);
  execBox->fill (alwaysShow);
  quit = new Button ("Quit");
  quit->connectSignal ("clicked", orxonoxGUI->orxonox_gui_quit);
  execBox->fill (quit);

  execFrame->fill (execBox);
}

/**
   \brief Return the Frame
   \return Returns the Exec-frame
*/
Widget* OrxonoxGuiExec::getWidget ()
{
  return execFrame;
}

/* FILE HANDLING */

/**
   \brief Sets the location of the configuration File.\n
   * The name will be parsed from ~/ to /home/[username] on unix and c:/Documents and Settings/username/Settings/ on Windows
   \param filename the location of the configFile
*/
void OrxonoxGuiExec::setFilename (char* filename)
{
  char* buffer = (char*) malloc (512*sizeof(buffer));
  sprintf (buffer, "%s", filename);
  if (!strncmp (buffer, "~/", 2))
  {
#ifdef __WIN32__
    sprintf (configFile, "%s/%s", getenv ("USERPROFILE"), buffer+2);
#else
    sprintf (configFile, "%s/%s", getenv ("HOME"), buffer+2);
#endif
  }
  else if (buffer)
    sprintf(configFile, "%s", buffer);
  delete buffer;
}

/**
   \brief checks if a option should be saved.
   \return 1 if it should 0 if not/
*/
int OrxonoxGuiExec::shouldsave ()
{
  return ( static_cast<Option*>(saveSettings)->value);
}

/** 
    \brief Saves the configuration-file to the Disk.\n
    this Function only opens and closes the file, in between OrxonoxGuiExec::writeFileText (Widget* widget) will execute the real writing process.
    \param widget from which Widget on should be saved.
*/
void OrxonoxGuiExec::writeToFile (Widget* widget)
{
  CONFIG_FILE = fopen (configFile, "w");
  if (CONFIG_FILE)
    writeFileText (widget);
  fclose (CONFIG_FILE);
}

/**
   \brief Actually writes into the configuration file to the disk.
   \param widget from which Widget on should be saved.
*/
void OrxonoxGuiExec::writeFileText (Widget* widget)
{
  if (widget->is_option <0)
    if (strcmp (static_cast<Packer*>(widget)->getGroupName(), ""))
      fprintf (CONFIG_FILE, "[%s]\n", static_cast<Packer*>(widget)->getGroupName());

  if (widget->is_option >= 1)
    if  (strcmp (static_cast<Option*>(widget)->flag_name, "") || strcmp (static_cast<Option*>(widget)->flag_name_short, ""))
      {
	char Buffer[256];
	char* space2under;
	sprintf (Buffer, "%s", static_cast<Option*>(widget)->label);
	if (strchr (Buffer, '_'))
	  cout << "Warning Optionname" << Buffer << " is not Valid for Saving, because it includes an underscore" << endl;
	while (space2under = strchr(Buffer, ' '))
	  {
	    sprintf (space2under, "_%s", space2under+1);
	  }
	fprintf (CONFIG_FILE, "%s = %i\n", Buffer, static_cast<Option*>(widget)->value);
      }
  if (widget->is_option<0)
    {
      writeFileText (static_cast<Packer*>(widget)->down);
    } 
  
  if (widget->next != NULL)
    writeFileText (widget->next);
}

/**
   \brief Reads in Configuration Data.
   \param widget from which Widget on should be saved.
*/
void OrxonoxGuiExec::readFromFile (Widget* widget)
{
  CONFIG_FILE = fopen (configFile, "r");
  if (CONFIG_FILE)
    {
      char Buffer[256] = "";
      char Variable[256]= "";
      int Value;
      while (fscanf (CONFIG_FILE, "%s", Buffer) != EOF)
	{
	  if (!strcmp (Buffer, "="))
	    {
	      char* under2space;
	      while (under2space = strchr(Variable, '_'))
		{
		  sprintf (under2space, " %s", under2space+1);
		}
	      
	      fscanf (CONFIG_FILE, "%s", Buffer);
	      Value = atoi(Buffer);
	      readFileText (widget, Variable, Value);
	      sprintf (Variable, "");
	    }
	  sprintf (Variable, "%s", Buffer);
	}
      widget->walkThrough(widget->setOptions);
    }
}

/**
   \brief Maps Confugurations to the Options.
   \param widget which widget downwards
   \param variableName the name of the Variable that should be set up.
   \param variableValue the Value of the Variable that should be set up
*/
void OrxonoxGuiExec::readFileText (Widget* widget, char* variableName, int variableValue)
{
  if (widget->is_option >= 1)
    if (!strcmp (static_cast<Option*>(widget)->label, variableName))
	static_cast<Option*>(widget)->value = variableValue;

  switch (widget->is_option)
    {
    case -1:
      readFileText (static_cast<Container*>(widget)->down, variableName, variableValue);
      break;
    case -2:
      readFileText (static_cast<Box*>(widget)->down, variableName, variableValue);
      break;
    } 

  if (widget->next != NULL)
    readFileText (widget->next, variableName, variableValue);
}

/**
   \brief Starts ORXONOX. (not really implemented yet, but the function is there.\n
   This is a Signal and can be executed through Widget::signal_connect
   \param widget the widget that executed the start command
   \param data additional data
*/
gint startOrxonox (GtkWidget *widget, Widget* data)
{
  cout << "Starting Orxonox" <<endl;
}
